@@ -1,3 +1,21 @@
+/* Process handling for Windows
+Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+This file is part of GNU Make.
+
+GNU Make is free software; you can redistribute it and/or modify it under the
+terms of the GNU General Public License as published by the Free Software
+Foundation; either version 3 of the License, or (at your option) any later
+version.
+
+GNU Make is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program.  If not, see <http://www.gnu.org/licenses/>.  */
+
+#include <config.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
@@ -19,7 +37,7 @@ int _cdecl compare(const void *a1, const void *a2)
 	return _stricoll(*((char**)a1),*((char**)a2));
 }
 bool_t
-arr2envblk(char **arr, char **envblk_out) 
+arr2envblk(char **arr, char **envblk_out)
 {
 	char **tmp;
 	int size_needed;
@@ -59,5 +77,7 @@ arr2envblk(char **arr, char **envblk_out)
 		ptr += strlen(tmp[arrcnt]) + 1;
 		arrcnt++;
 	}
+
+	free(tmp);
 	return TRUE;
 }
